;*******************************************************************************
;                                                                              *
;    Filename:	    an-mtr.asm                                                 *
;    Date:          May-2020                                                   *
;    File Version:  2.0                                                        *
;    Compiler: MPASMWIN (v5.86) (MicroChip)			               *
;                                                                              *
;    Original Author: Ronald Dekker                                            *
;    Company: dos4ever.com                                                     *
;    Modified: Bill van Dijk                                                   *
;                                                                              *
;*******************************************************************************
;*******************************************************************************
;                                Notes:                                        *
;                                                                              *
;The second timing pulse is derived from the AC power input, filtered,         *
;shaped, and arrives as a 50/60Hz square pulse with a negative period          *
;of about 5.39 ms duration, it triggers EXT INT on the neg. edge. In order to  *
;to accomodate different locales jumper JP9 set the net frequency. When the    *
;jumper is absent, the clock expects a 60Hz frequency, when a jumper is        *
;installed 50Hz is expected.						       *
;                                                                              *
;Ronald Dekker originally provided three color transition options, but required*
;the software to be recompiled to select a different option. A 2 section slide *
;switch was added to enable selection on the fly. When both switches are set to*
;zero, the lights are off.						       *
;								               *
;The calibration loop is a closed loop, accessed by holding the button         *
;down while powering up. Use the pots on the board to set the meters           *
;so that the min, center, and max positions match the scale markings. Exit the *
;calibration loop by cycling the power. This only needs to be done once.       *
;								               *
;timer0 (8-bit) is configured to generate an interrupt every 200us             *
;At 4MHz oscillator frequency, the internal clock runs at 1 mHz = 1 us         *
;the prescaler is set at 1:4 so that counting period of timer0 is 4us          *
;this requires 200/4 = 50 counts. This makes the reload value for              *
;TMR0=256-50 = 206 or 0xCE	                 		               *
;								               *
;Timer1 is a 16 bit timer, set to count 3ms to validate the EXT INT pulse.     *
;Once an EXTernal INTerrupt is detected, timer1 is started and 3ms later the   *
;pulse is verified to see if it is still valid before being accepted and       *
;processed. The thought is that any line noise which may make it through the   *
;front end filters would not last longer than 3ms, and as such it should be    *
;virtually impossible to get a false time trigger. In order to get a 3ms       *
;overflow: 0xFFFF - 0x0BB8 = 0xF447. The high byte is therefore loaded with    *
;0xF4, and the low byte with 0x47.					       *
;								               *
;subroutine: pwm (called every 200us from timer 0 interrupt overflow)          *
;This peace of program generates the pulse width modulated output              *
;which drives the analog meters. The program consists of three almost          *
;identical subparts: one for the seconds, one for the minutes and one          *
;for the hours. Lets for example look at the last loop which is for            *
;the hours. This loop counts down counter HRSPWM from 12 or 24 to 0            *
;every call. When the counter reaches 0 it resets the output and reloads       *
;the counter. Before returning the counter value is compared to HRS.           *
;When they are equal the oputput is set. Same logic for the minutes and seconds*
;								               *
;subroutine: rnd_col (called every 200us from routine "tmrint")		       *
;This single subroutine displays a sequence of colors from a table on a LED.   *
;The routine generates a smooth transistion between the colors. The mixing     *
;of the colors, in the program called COLA and COLB, is done by multiplexing   *
;between the two colors. The initial duty-cycle in this switching is	       *
;such that only COLA is shown. The duty-cycle gradually changes so that at the *
;and of a transition only COLB is shown. This process is called a transition.  *
;A transition consists of 32 or 64 steps. A single multiplex cycle, whereby    *
;alternately COLA and COLB are shown, is called a cycle. A cycle has to be     *
;shorter than 20ms in order for the eye to percieve the two colors as one      *
;mixed color. The ratio between COLA and COLB is determined by variable	       *
;A2B_RAT in such a way that when A2B_RAT=00 only COLB is shown.		       *
;COLA and COLB are generated using pulse-width modulation. For each color      *
;2 bits have been used, so that one color can be represented by one byte.      *
;The format for a color is xxRR GGBB. For example 0011 0000 is full intensity  *
;red, 0001 0000 represents 33% intensity red, xx11 1111 is white. The code     *
;1111 1111 represents the end of the table. One pulse width modulation cycle   *
;is called a burst. With 2 bits, a burst consists of 4 levels. Each cycle      *
;consists of either 32 or 62 bursts.					       *
;*******************************************************************************

;compiler directives:
 list      p=16f628A     ; list directive to define processor
 #include <p16f628a.inc> ; processor specific variable definitions
 errorlevel  -302        ; suppress message 302 from list file


 __CONFIG _INTOSC_OSC_NOCLKOUT & _WDTE_OFF & _PWRTE_ON & _MCLRE_ON & _BODEN_OFF & _LVP_OFF & _CPD_OFF & _CP_OFF

;RAM declarations starting at 0x20:
 CBLOCK 0x20
    BITS1	;LedsOFF,-----,GOTIME,-----,-----,----,LIGHTS,AB
    W_TEMP	;interrupt save for W
    STATUS_TEMP ;interrupt save for STATUS
    COLOR	;current color code
    LCNT	;level counter
    LTMP	;temp for the level routine
    COLA	;color A
    COLB	;color B
    A2B_RAT	;ratio between COLB and COLA (00=colB)
    NBURST	;burst counter (number of burst in a cycle)
    NCYCLE	;cycle counter (number of cycles in a transition step)
    INDX	;pointer in the color table
    M_FREQ	;counts down 60 or 50HZ periods for a second
    SEC		;seconds
    MIN		;minutes
    HRS		;hours
    SECPWM	;pwm counter for seconds
    MINPWM	;pwm counter for minutes
    HRSPWM	;pwm counter for hours
    MAINS	;stores mains frequency
    TBL_CALL	;0, 1, or 2 (which color table to call)
    TblData	;returned value from table read
    d1		;delay routine variable
    d2		;delay routine variable
    d3		;delay routine variable
    d4		;delay routine variable
    d5		;delay routine variable
 ENDC

;THE CONSTANTS
NCYCLE_REL EQU 0x06 ;number of cycles making up one transistion stepthis in
;principle determines the time every transistiontakes. 01H is highest speed.
;Increase for slower transistions.
NBURST_MAX EQU 0x1F  ;number of bursts in a cycle, and steps in a transistion
MAXHRS	   EQU d'12' ;12 hour clock
T0_LOAD	   EQU 0xCE  ;value to be loaded in timer 0
T1_LLOAD   EQU 0x47  ;value to be loaded in T1 low on reset (3ms overflow)
T1_HLOAD   EQU 0xF4  ;value to be loaded in T1 high on reset (3ms overflow)

;The flag Bits BITS1
AB	 EQU 0x00    ;if 1, color A is on
LIGHTS	 EQU 0x01    ;if 1, run the LED light routine
GOTIME   EQU 0x05    ;clock is running when set
LedsOFF	 EQU 0x07    ;don't light the LEDS when set

 #define PULSE	RB0	;input from mains frequency parser
 #define BUTTON RB4	;input from function button
 #define JUMPER RB5	;mains frequency select jumper (open = 60Hz, closed 50)
 #define SW2	RA6	;color pattern table select switch
 #define SW1	RA7	;color pattern table select switch
 #define LEDR	RA0	;red LED output
 #define LEDG	RA1	;green LED output
 #define LEDB	RA2	;blue LED output
 #define Mhr	RB1	;meter, hours
 #define Mmin	RB2	;meter output minutes
 #define Msec	RB3	;meter output seconds


;===============================================================================
;********************** PROGRAM CODE SECTION ***********************************
;===============================================================================


;***************************RESET CODE******************************************
;*******************************************************************************
 ORG  0x00		; processor reset vector, execute code from start
 nop
 GOTO _START


;************************* INTERRUPT CODE **************************************
;*******************************************************************************
;>>>>>-------->SAVE ENVIRONMENT AT THE MOMENT OF INTERRUPT
 ORG 0x04		;start at interrupt vector location 0x04
 movwf  W_TEMP		;copy w to temp register
 swapf  STATUS,W	;swap status to be saved in W
 movwf  STATUS_TEMP	;save status register
;>>>>>>-------->CHECK IF TIMER 0 OVERFLOWED
 btfss	INTCON,T0IF	;if timer 0 overflowed (200ms) do PWM and color stuff
 goto	_Done_TMR0	;if not, goto next interrupt
 bcf	INTCON,T0IF	;reset interrupt flag
 movlw  T0_LOAD		;reload value for TIMER 0
 movwf  TMR0		;
 call   _PWM		;do the pwm routine for the analog meters now
 btfsc  BITS1,LedsOFF	;skips LED processing if set
 goto   _DontColor	;
 btfss  PORTA,SW1	;if both switches are set to 0
 goto   _DoColor	;read the table if set
 btfss	PORTA,SW2	;and  don't light any of the LEDs
 goto	_DoColor	;read the table if set
 goto	_DontColor	;skip the LED color routine
_DoColor
 bsf	BITS1,LIGHTS	;set the lights on flag
 goto	_Done_TMR0	;done
_DontColor
 bcf	PORTA,LEDR
 bcf	PORTA,LEDG
 bcf	PORTA,LEDB
_Done_TMR0
;>>>>>-------->CHECK IF EX_INT INTERRUPT IS TRIGGERED, if yes, start timer1
 btfss	INTCON,INTF	;check if external interrupt has tripped
 goto	_Skip_EX_INT    ;if not, goto next interrupt
 bcf	INTCON,INTF	;reset external interrupt flag
 movlw	T1_LLOAD	;load timer 1 for 25ms overflow
 movwf	TMR1L		;
 movlw	T1_HLOAD	;
 movwf	TMR1H		;
 bsf	T1CON,TMR1ON	;start timer 1 countdown
_Skip_EX_INT
;>>>>>-------->CHECK IF TIMER 1 OVERFLOWED (test for valid mains pulse)
 btfss	PIR1,TMR1IF	;check for timer 1 overflow (25ms delay, retest ext int)
 goto	_Skip_TMR1	;if not, goto next interrupt
 bcf	T1CON,TMR1ON	;stop timer 1 countdown
 bcf	PIR1,TMR1IF	;reset Timer 1 interrupt flag
 btfsc	PORTB,PULSE	;check if mains pulse pulled low by opto coupler is stil valid
 goto	_Skip_TMR1	;if not, it was a false trigger
 call	_PULSE_CNT	;check if we reached a new second yet
_Skip_TMR1
;>>>>>-------->DONE, RESTORE ENVIRONMENT TO THE MOMENT OF INTERRUPT
 swapf  STATUS_TEMP,W	;swap a copy of STATUS register into W
 movwf  STATUS		;restore pre-isr STATUS register contents
 swapf	W_TEMP,F	;swap data in W_TEMP
 swapf  W_TEMP,W	;restore original contents of the saved W-TEMP to W
 retfie
;********************** END OF INTERRUPT CODE **********************************


;************************ CONFIGURE DEVICE *************************************
;*******************************************************************************
_START
;initialize PORTA and PORTB
 clrf	  PORTA
 clrf	  PORTB
 movlw    b'00000111'	; (0x07)
 movwf    CMCON		;turn comparators off, and set pins to IO
 banksel  TRISA		;switch to bank 1
 movlw    b'11110000'	; (0xF0)
 movwf    TRISA		;PORTA: in, in, in, in, (test)out, (BLUE)out, (GREEN)out, (RED)out.
 movlw    b'11110001'	; (0xF1)
 movwf    TRISB		;PORTB: in, in, in, in, meter, meter, meter, in.
 banksel  PORTB		;return to bank 0
 CLRF	  PCLATH	;Ensure page bits are cleared
;initialize timer 0 (8BIT timer with 8BIT prescaler, set to overflow each 200ms))
 clrf	  TMR0		;clear timer 0 and its prescaler
 banksel  OPTION_REG	;switch to bank 1
 movlw    b'10000001'	;timer0 on internal clock, presc 1:4 INT on falling edge, PORTB pullups disabled
 movwf    OPTION_REG	; (0x81)
 banksel  TMR0		;return to bank 0
 movlw    0xCE		;load TMR0
 movwf    TMR0		;
 movlw    b'11110000'	;set GIE, timer1, timer0, and RB0/INT interrupt on
 movwf    INTCON	; (0xF0)
;initialize timer 1 (16BIT) and start (used to "debounce" EXT INT )
 clrf	  T1CON		;No prescaler, int oscilator, rising edge, off for now
 banksel  PIE1		;peripheral interrupt control register
 bsf	  PIE1,TMR1IE	;set timer 1 interrupt enable bit, bit0
 banksel  TMR0		;return to bank 0
 bcf	  PIR1,TMR1IF	;clear the timer 1 overflow flag
 movlw	  T1_HLOAD	;
 movwf	  TMR1H		;Load timer 1 high byte
 movlw	  T1_LLOAD	;
 movwf	  TMR1L		;Load timer 1 low byte
;initialize various program variables and bits
 movlw	  d'60'		;initialize the pwm counter for seconds and minutes to 60
 movwf	  SECPWM	;
 movwf    MINPWM	;
 movlw	  MAXHRS	;initialize the pwm counter for hours to 12 or 24
 movwf	  HRSPWM	;
 movlw    0x3F		;initialize level counter
 movwf    LCNT	        ;
 clrf	  NBURST	;
 movlw    NCYCLE_REL	;
 movwf    NCYCLE	;
 clrf	  A2B_RAT	;
 clrf	  INDX		;set table offset to 0
 call	  _ReadTable	;table read
 movf	  TblData,w	;save result
 movwf    COLA		;
 incf	  INDX,F	;set table offset to 1
 call	  _ReadTable	;table read
 movf	  TblData,w	;save result
 movwf    COLB		;
 incf     INDX,F	;set table offset to 2
 movlw	  d'60'		;load 60 in W
 btfss	  PORTB,JUMPER	;check jumper, 60Hz if not, 50Hz if installed
 movlw	  d'50'		;replace with 50 if jumper is installed
 movwf	  MAINS		;store correct mains frequency in storage register
 movwf    M_FREQ	;store correct mains frequency in working register
;**********************END OF CONFIGURATION CYCLE-******************************


;*****************CALIBRATION LOOP - (INTERRUPTS ARE ACTIVE)********************
;*******************************************************************************
_main
 ;******SET METERS TO ZERO******************
 btfsc    PORTB,BUTTON	;if key is pressed at start-up run calibration program
 goto	  _SetTime	;otherwise goto clock set routine (key normally high)
 call	  _DELAY15	;wait 25ms to debounce the button
 btfss    PORTB,BUTTON	;wait here to make sure the button is released
 goto	  $-1
 bcf	  BITS1,GOTIME  ;stop the mains frequency based update of the clock
 bsf	  BITS1,LedsOFF	;don't light the LEDs yet during calibration
_CalLoop
 clrf	  SEC		;sec,min,hrs 00:00:00 (minimum scale)
 clrf	  MIN		;
 clrf	  HRS		;
;*******SET METERS TO HALF SCALE************
 btfsc    PORTB,BUTTON	;wait here until button is pushed
 goto	  $-1
 call	  _DELAY15	;wait 25ms to debounce the button
 btfss    PORTB,BUTTON	;wait here until button is released
 goto	  $-1
 movlw    d'30'		;sec:min:hrs 30:30:6/12 (half scale)
 movwf    SEC		;
 movwf    MIN		;
 movlw	  MAXHRS/2	;load half of max. hour scale in W
 movwf    HRS		;
;*******SET METERS TO FULL SCALE*************
 btfsc    PORTB,BUTTON	;wait here until button is pushed
 goto	  $-1
 call	  _DELAY15	;wait 25ms to debounce the button
 btfss    PORTB,BUTTON	;wait here until button is released
 goto	  $-1
 movlw    d'59'		;sec:min:hrs 60:60:12/24 (full scale)
 movwf    SEC		;
 movwf    MIN		;
 movlw    MAXHRS-1	;
 movwf    HRS		;
 btfsc    PORTB,BUTTON	;wait here until button is pushed
 goto	  $-1
 call	  _DELAY15	;wait 25ms to debounce the button
 btfss    PORTB,BUTTON	;wait here until button is released
 goto	  $-1
 goto	  _CalLoop	;calibration loop, power cycle for normal clock function
;+++END OF CLOSED CALIBRATION LOOP+++POWER CYCLE TO EXIT TO NORMAL OPERATION++++


;************************TIME SETTING LOOP**************************************
;*******************************************************************************
;get ready to set time
_SetTime
 bcf	  BITS1,GOTIME  ;stop the mains frequency based update of the clock
 bsf	  BITS1,LedsOFF	;don't light the LEDs yet during calibration
 clrf	  HRS		;set time to 00:00:00
 clrf	  MIN		;
 clrf	  SEC		;
_SetHrLoop  ;set hours loop
 movlw 0xA8
 movwf d3
 movlw 0x45
 movwf d4
 movlw	0x03
 movwf d5
_HrLoop
 btfsc	  PORTB,BUTTON	;continue this loop until the button is pushed
 goto	  $+5
 call	  _DELAY15	;debounce
 btfss	  PORTB,BUTTON	;wait until button is released again
 goto	  $-1
 goto	  _SetMinLoop	;go set minutes next
 decfsz   d3,F
 goto     _HrLoop
 decfsz   d4,F
 goto     _HrLoop
 decfsz   d5,F
 goto     _HrLoop
 incf	  HRS,F		;increment HRS until HRS=24
 movlw	  MAXHRS	;reset to zero on 13 or 25
 xorwf	  HRS,W
 btfsc    STATUS,Z	;
 clrf	  HRS		;reset HRS otherwise
 goto	  _SetHrLoop
_SetMinLoop ;set minutes loop
 movlw 0xA8
 movwf d3
 movlw 0x45
 movwf d4
 movlw 0x02
 movwf d5
_MinLoop
 btfsc	  PORTB,BUTTON	;continue this loop until the button is pushed
 goto	  $+5
 call	  _DELAY15	;debounce
 btfss	  PORTB,BUTTON	;wait until button is released again
 goto	  $-1
 goto	  _DoneSet	;done setting time
 decfsz   d3,f
 goto     _MinLoop
 decfsz   d4,f
 goto     _MinLoop
 decfsz   d5,f
 goto     _MinLoop
 incf	  MIN,F		;increment MIN until MIN=60
 movf	  MIN,W		;
 xorlw    0x3C		;count up to 60 minutes, than reset to zero at 60
 btfsc    STATUS,Z	;
 clrf	  MIN		;reset MIN otherwise
 goto	  _SetMinLoop
_DoneSet    ;start normal clock operation
 movf	  MAINS,W	;store mains frequency
 movwf    M_FREQ	;start with a nice new second
 bsf	  BITS1,GOTIME  ;start the clock
 bcf	  BITS1,LedsOFF ;OK to light the LEDs
 goto	  _MainLoop
;**********************end of time setting loop*********************************


;********************MAIN PROGRAM INNER LOOP************************************
;*******************************************************************************
;loop here forever or until key is pressed, then goto set time again.
_MainLoop
;+++++++++++++CHECK IF BUTTON WAS PUSHED, if so, set the time again.
 btfsc	  BITS1,LIGHTS	;
 call	  _rnd_col	;process the ambi-light feature
 btfsc    PORTB,BUTTON	;check if button is pushed
 goto	  _MainLoop	;loop here for ever
 call	  _DELAY15	;wait 25ms to debounce the button
 btfss    PORTB,BUTTON	;hold here until button is released
 goto	  $-1		;
 goto	  _SetTime	;go set the time again
;*******************END OF INNER MAIN PROGRAM LOOP******************************


;*****************CHECK IF WE HAVE FOUND A NEW SECOND***************************
;*******************************************************************************
_PULSE_CNT
 btfss    BITS1,GOTIME  ;clock update is stopped during calibration and time set
 return			;skip time advance if flag is not set.
 decfsz	  M_FREQ,F	;lower M_FREQ by one
 return			;exit if not yet zero
 movf	  MAINS,W
 movwf	  M_FREQ	;reload M_FREQ
 incf	  SEC,F		;increment SEC until SEC=60
 movf	  SEC,W		;
 xorlw    d'60'		;60 seconds
 btfss    STATUS,Z	;
 return			;
 clrf	  SEC		;reset SEC
 incf	  MIN,F		;increment MIN until MIN=60
 movf	  MIN,W		;
 xorlw    d'60'		;60 minutes
 btfss    STATUS,Z	;
 return			;
 clrf	  MIN		;reset MIN
 incf	  HRS,F		;increment HRS until HRS=24
 movf	  HRS,W		;
 xorlw    MAXHRS	;max. hour display
 btfss    STATUS,Z	;
 return			;
 clrf	  HRS		;
 return			;
;************************END NEW SECOND LOOP************************************


;********************METER DISPLAY CONTROL**************************************
;*******************************************************************************
_PWM
;>>>>>------> process seconds display
 decfsz   SECPWM,F	;decrease second pwm counter by 1
 goto	  _pwm_1	;skip reload if not yet 0
 movlw    0x3C		;reload pwm counter with d'60 and clear sec output M0
 movwf    SECPWM	;
 bcf	  PORTB,Msec	;Meter, seconds
_pwm_1
 movf	  SECPWM,W	;set output M0 if pwm counter = sec
 xorwf    SEC,W		;
 btfsc    STATUS,Z	;
 bsf	  PORTB,Msec	;Meter, seconds
;>>>>>------> process minutes display
 decfsz   MINPWM,F	;decrease minute pwm counter by 1
 goto	  _pwm_2	;skip reload if not yet 0
 movlw    0x3C		;reload pwm counter with 60dec and clear min output M1
 movwf    MINPWM	;
 bcf	  PORTB,Mmin	;
_pwm_2
 movf	  MINPWM,W	;set output M1 if pwm counter = min
 xorwf    MIN,W		;
 btfsc    STATUS,Z	;
 bsf	  PORTB,Mmin	;Meter, minutes
;>>>>>------> process hours display
 decfsz   HRSPWM,F	;decrease hour pwm counter by 1
 goto	  _pwm_3	;skip reload if not yet 0
 movlw    MAXHRS	;reload pwm counter with MAXHRS
 movwf    HRSPWM	;
 bcf	  PORTB,Mhr	;Meter, hours
_pwm_3
 movf	  HRSPWM,W	;set output M2 if pwm counter = hrs
 xorwf    HRS,W		;
 btfsc    STATUS,Z	;
 bsf	  PORTB,Mhr	;Meter, hours
 return
;***********************END METER CONTROL***************************************


;***********************TABLE READ ROUTINE**************************************
;*******************************************************************************
_ReadTable
  btfsc	PORTA,SW1	;check slide switch (port reads 1 when switch is off)
  goto	_ReadB	    	;skip if off
  movlw	HIGH _TBL_A	;load table high address in W
  movwf	PCLATH		;and load it in PCLATH
  movf	INDX,W		;load table off-set in W
  call	_TBL_A		;get data
  movwf	TblData		;save data retruned from the table read
  return
_ReadB
  btfsc	PORTA,SW2	;check slide switch (port reads 1 when switch is off)
  goto	_ReadC		;skip if off
  movlw	HIGH _TBL_B	;load table high address in W
  movwf	PCLATH		;and load it in PCLATH
  movf	INDX,W		;load table off-set in W
  call	_TBL_B		;get data
  movwf	TblData		;save data
  return
_ReadC
  movlw	HIGH _TBL_C	;load table high address in W
  movwf	PCLATH		;and load it in PCLATH
  movf	INDX,W		;load table off-set in W
  call	_TBL_C		;get data
  movwf	TblData		;save data
 return
;**************************END TABLE READ***************************************



;****************LED COLOR TRANSITION CODE**************************************
;*******************************************************************************
_rnd_col
 bcf	  BITS1,LIGHTS	;clear the run lights flag
 movf     NBURST,W	;First see if in this burst we have
 xorwf    A2B_RAT,W	;to show COLA or COLB. The switch
 btfsc    STATUS,Z	;from COLA to COLB in a cycle is made
 bsf	  BITS1,AB	;when A2B_RAT=NBURST
 movf     COLB,W	;COLOR becomes the actual color
 movwf    COLOR		;
 movf     COLA,W	;
 btfsc    BITS1,AB	;
 movwf    COLOR		;
 movf     LCNT,W	;This part generates the levels in a burst
 xorlw    0x3F		;by switching the LEDs on (and off).
 btfss    STATUS,Z	;LEDS of at beginning of a new burst
 goto     _rnd_col_1	;
 bcf	  PORTA,LEDR	;(RA0) turn the LEDs of
 bcf	  PORTA,LEDG	;(RA1)
 bcf	  PORTA,LEDB	;(RA2)
_rnd_col_1
 movf     COLOR,W	;xor the color with the level
 xorwf    LCNT,W	;counter and save in LTMP
 movwf    LTMP		;
 andlw    0x03		;single out color 1
 btfsc    STATUS,Z	;skip if color.neq.counter
 bsf	  PORTA,LEDR	;set color 1
 movf     LTMP,W	;single out color 2
 andlw    0x0C		;
 btfsc    STATUS,Z	;skip if color.neq.counter
 bsf	  PORTA,LEDG	;set color 2
 movf     LTMP,W	;single out color 3
 andlw    0x30		;
 btfsc    STATUS,Z	;skip if color.neq.counter
 bsf	  PORTA,LEDB	;set color 3
 movlw    0x15		;decrement the level counter with 15H
 subwf    LCNT,W	;LCNT-W -> W
 movwf    LCNT		;
 btfss    STATUS,Z	;if zero skip
 return			;if not zero return
 movlw    0x3F		;reset level counter
 movwf    LCNT		;
 incf     NBURST,F	;in this part we count the number of
 movf     NBURST,W	;bursts in one cycle. Note that by
 xorlw    NBURST_MAX	;doing it this way, NBURST=1FH will
 btfss    STATUS,Z	;immediately result in a reset of
 return			;of NBURST, so that COLA will never
 clrf     NBURST	;be displayed.
 bcf	  BITS1,AB	;
 decfsz   NCYCLE,F	;in this small part, the number of
 return			;cycles that make up one transition
 movlw    NCYCLE_REL	;step are counted. By modifying
 movwf    NCYCLE	;NCYCLE_REL the transition speed may be modified
 incf     A2B_RAT,F	;after each transition step A2B_RAT
 movf     A2B_RAT,W	;is incremented. This for every
 xorlw    1+NBURST_MAX  ;transistion step modifies the
 btfss    STATUS,Z	;COLA/COLB ratio. When we have gone
 return			;through all the possible transition
 clrf     A2B_RAT	;step the next color is loaded.
 movf     COLB,W	;COLB now becomes the starting color
 movwf    COLA		;COLA, and a new color from the table
 call	  _ReadTable
 movf	  TblData,w	;
 movwf    COLB		;if FF is found the pointer "loops
 incf     INDX,F	;around" to the beginning of the
 call	  _ReadTable
 movf	  TblData,W	;
 xorlw    0xFF		;
 btfss    STATUS,Z	;
 return			;
 clrf     INDX		;
 return			;return
;****************END LED COLOR TRANSITION***************************************


;******************DELAY LOOP***************************************************
;*******************************************************************************
_DELAY15    ; Delay = 0.015 seconds
; Clock frequency = 4 MHz
; Actual delay = 0.015 seconds = 15000 cycles
 movlw 0x78
 movwf d1
 movlw 0x14
 movwf d2
_LOOP15
 decfsz d1,F
 goto _LOOP15
 decfsz d2,F
 goto _LOOP15
 nop
 return
;******************END DELAY LOOP***********************************************


;****************LED COLOR TRANSITION data tables*******************************
;*******************************************************************************
 org 0x602 ;(Page 6) placed high to prevent the tables from stradeling
;or crossing a PCL 256 byte boundary line

;GROUP1 only primary colors
;GROUP2 include also two primary colors
;GROUP3 include also three primary colors

;This GROUP1 table is  random list of primary colors.
_TBL_A		addwf	PCL,F		;
		retlw	0x03		;00
		retlw	0x30		;
		retlw	0x0C		;
		retlw	0x20		;
		retlw	0x02		;04
		retlw	0x10		;
		retlw	0x03		;
		retlw	0x10		;
		retlw	0x20		;08
		retlw	0x02		;
		retlw	0x04		;
		retlw	0x08		;
		retlw	0x01		;0C
		retlw	0x0C		;
		retlw	0x30		;
		retlw	0x08		;
		retlw	0x30		;10
		retlw	0x10		;
		retlw	0x01		;
		retlw	0x03		;
		retlw	0x0C		;14
		retlw	0x20		;
		retlw	0x02		;
		retlw	0x04		;
		retlw	0x0C		;18
		retlw	0x08		;
		retlw	0x20		;
		retlw	0x04		;
		retlw	0x10		;1C
		retlw	0x02		;
		retlw	0x30		;
		retlw	0x01		;
		retlw	0x03		;20
		retlw	0x30		;
		retlw	0x01		;
		retlw	0x02		;
		retlw	0x10		;24
		retlw	0x20		;
		retlw	0x0C		;
		retlw	0x08		;
		retlw	0x04		;28
		retlw	0x10		;
		retlw	0x03		;
		retlw	0x20		;
		retlw	0x30		;2C
		retlw	0x08		;
		retlw	0x01		;
		retlw	0x04		;
		retlw	0x0C		;30
		retlw	0x02		;
		retlw	0x0C		;
		retlw	0x02		;
		retlw	0x20		;34
		retlw	0x03		;
		retlw	0x30		;
		retlw	0x01		;
		retlw	0x04		;38
		retlw	0x10		;
		retlw	0x08		;
		retlw	0x01		;
		retlw	0x03		;3C
		retlw	0x04		;
		retlw	0x02		;
		retlw	0x10		;
		retlw	0x08		;40
		retlw	0xFF



;this GROUP2 table is a list of colors that consist of one or two primary colors
_TBL_B		addwf	PCL,F		;
		retlw	0x3C		;00
		retlw	0x30		;
		retlw	0x03		;
		retlw	0x0B		;
		retlw	0x0C		;04
		retlw	0x38		;
		retlw	0x23		;
		retlw	0x22		;
		retlw	0x04		;08
		retlw	0x03		;
		retlw	0x0E		;
		retlw	0x1C		;
		retlw	0x20		;0C
		retlw	0x0D		;
		retlw	0x33		;
		retlw	0x08		;
		retlw	0x0A		;10
		retlw	0x33		;
		retlw	0x05		;
		retlw	0x31		;
		retlw	0x08		;14
		retlw	0x14		;
		retlw	0x08		;
		retlw	0x20		;
		retlw	0x02		;18
		retlw	0x10		;
		retlw	0x34		;
		retlw	0x02		;
		retlw	0x06		;1C
		retlw	0x23		;
		retlw	0x0E		;
		retlw	0x0D		;
		retlw	0x06		;20
		retlw	0x0F		;
		retlw	0x14		;
		retlw	0x09		;
		retlw	0x13		;24
		retlw	0x02		;
		retlw	0x32		;
		retlw	0x07		;
		retlw	0x24		;28
		retlw	0x21		;
		retlw	0x18		;
		retlw	0x01		;
		retlw	0x0F		;2C
		retlw	0x28		;
		retlw	0x10		;
		retlw	0x2C		;
		retlw	0x03		;30
		retlw	0x3C		;
		retlw	0x08		;
		retlw	0x10		;
		retlw	0x0C		;34
		retlw	0x0A		;
		retlw	0x28		;
		retlw	0x34		;
		retlw	0x0B		;38
		retlw	0x13		;
		retlw	0x21		;
		retlw	0x04		;
		retlw	0x32		;3C
		retlw	0x10		;
		retlw	0x02		;
		retlw	0x31		;
		retlw	0x0C		;40
		retlw	0x30		;
		retlw	0x01		;
		retlw	0x04		;
		retlw	0x30		;44
		retlw	0x22		;
		retlw	0x11		;
		retlw	0x01		;
		retlw	0x03		;48
		retlw	0x38		;
		retlw	0x05		;
		retlw	0x0C		;
		retlw	0x09		;4C
		retlw	0x20		;
		retlw	0x12		;
		retlw	0x20		;
		retlw	0x2C		;50
		retlw	0x24		;
		retlw	0x1C		;
		retlw	0x30		;
		retlw	0x07		;54
		retlw	0x11		;
		retlw	0x18		;
		retlw	0x12		;
		retlw	0x04		;58
		retlw	0x01		;
		retlw	0x21		;
		retlw	0x08		;
		retlw	0x11		;5C
		retlw	0x0F		;
		retlw	0x20		;
		retlw	0x38		;
		retlw	0x20		;60
		retlw	0x06		;
		retlw	0x02		;
		retlw	0x28		;
		retlw	0x1C		;64
		retlw	0x07		;
		retlw	0x18		;
		retlw	0x22		;
		retlw	0x14		;68
		retlw	0x0B		;
		retlw	0x24		;
		retlw	0x02		;
		retlw	0x0E		;6C
		retlw	0x0B		;
		retlw	0x12		;
		retlw	0x05		;
		retlw	0x23		;70
		retlw	0x03		;
		retlw	0x12		;
		retlw	0x32		;
		retlw	0x08		;74
		retlw	0x02		;
		retlw	0x09		;
		retlw	0x10		;
		retlw	0x32		;78
		retlw	0x01		;
		retlw	0x1C		;
		retlw	0x08		;
		retlw	0x30		;7C
		retlw	0x05		;
		retlw	0x18		;
		retlw	0x03		;
		retlw	0x24		;80
		retlw	0x14		;
		retlw	0x08		;
		retlw	0x34		;
		retlw	0x0A		;84
		retlw	0x11		;
		retlw	0x09		;
		retlw	0x01		;
		retlw	0x0D		;88
		retlw	0x38		;
		retlw	0x0F		;
		retlw	0x0C		;
		retlw	0x2C		;8C
		retlw	0x22		;
		retlw	0x33		;
		retlw	0x04		;
		retlw	0x20		;90
		retlw	0x13		;
		retlw	0x28		;
		retlw	0x02		;
		retlw	0x01		;94
		retlw	0x3C		;
		retlw	0x0C		;
		retlw	0x3C		;
		retlw	0x07		;98
		retlw	0x10		;
		retlw	0x31		;
		retlw	0x20		;
		retlw	0x31		;9C
		retlw	0x0C		;
		retlw	0x06		;
		retlw	0x10		;
		retlw	0x23		;A0
		retlw	0x0E		;
		retlw	0x34		;
		retlw	0x13		;
		retlw	0x04		;A4
		retlw	0x30		;
		retlw	0x33		;
		retlw	0x0A		;
		retlw	0x21		;A8
		retlw	0x0C		;
		retlw	0x10		;
		retlw	0x01		;
		retlw	0x30		;AC
		retlw	0x04		;
		retlw	0x0D		;
		retlw	0x03		;
		retlw	0x30		;B0
		retlw	0x04		;
		retlw	0x03		;
		retlw	0xFF


 org 0x0702 ;(page 7)
;this group3 is a table with random colors containing 1,2 and 3 primary colors
_TBL_C		addwf	PCL,F		;
		retlw	0x33		;00
		retlw	0x29		;
		retlw	0x1C		;
		retlw	0x34		;
		retlw	0x02		;04
		retlw	0x24		;
		retlw	0x01		;
		retlw	0x18		;
		retlw	0x3F		;08
		retlw	0x1A		;
		retlw	0x03		;
		retlw	0x3E		;
		retlw	0x04		;0C
		retlw	0x0C		;
		retlw	0x30		;
		retlw	0x12		;
		retlw	0x09		;10
		retlw	0x0F		;
		retlw	0x12		;
		retlw	0x10		;
		retlw	0x02		;14
		retlw	0x30		;
		retlw	0x08		;
		retlw	0x26		;
		retlw	0x03		;18
		retlw	0x15		;
		retlw	0x38		;
		retlw	0x17		;
		retlw	0x10		;1C
		retlw	0x20		;
		retlw	0x04		;
		retlw	0x33		;
		retlw	0x19		;20
		retlw	0x34		;
		retlw	0x28		;
		retlw	0x23		;
		retlw	0x10		;24
		retlw	0x05		;
		retlw	0x11		;
		retlw	0x06		;
		retlw	0x2C		;28
		retlw	0x0E		;
		retlw	0x80		;
		retlw	0x0D		;
		retlw	0x22		;2C
		retlw	0x31		;
		retlw	0x2B		;
		retlw	0x07		;
		retlw	0x3D		;30
		retlw	0x04		;
		retlw	0x0B		;
		retlw	0x0A		;
		retlw	0x27		;34
		retlw	0x0F		;
		retlw	0x13		;
		retlw	0x09		;
		retlw	0x1B		;38
		retlw	0x18		;
		retlw	0x25		;
		retlw	0x3A		;
		retlw	0x22		;3C
		retlw	0x0C		;
		retlw	0x38		;
		retlw	0x30		;
		retlw	0x2D		;40
		retlw	0x20		;
		retlw	0x36		;
		retlw	0x1F		;
		retlw	0x3B		;44
		retlw	0x10		;
		retlw	0x37		;
		retlw	0x03		;
		retlw	0x21		;48
		retlw	0x20		;
		retlw	0x39		;
		retlw	0x32		;
		retlw	0x1C		;4C
		retlw	0x02		;
		retlw	0x2C		;
		retlw	0x2A		;
		retlw	0x11		;50
		retlw	0x08		;
		retlw	0x0C		;
		retlw	0x2F		;
		retlw	0x05		;54
		retlw	0x2E		;
		retlw	0x0E		;
		retlw	0x05		;
		retlw	0x28		;58
		retlw	0x3C		;
		retlw	0x16		;
		retlw	0x21		;
		retlw	0x0C		;5C
		retlw	0x1E		;
		retlw	0x02		;
		retlw	0x07		;
		retlw	0x24		;60
		retlw	0x06		;
		retlw	0x3C		;
		retlw	0x14		;
		retlw	0x0A		;64
		retlw	0x35		;
		retlw	0x10		;
		retlw	0x03		;
		retlw	0x20		;68
		retlw	0x0D		;
		retlw	0x14		;
		retlw	0x32		;
		retlw	0x01		;6C
		retlw	0x0B		;
		retlw	0x1D		;
		retlw	0x31		;
		retlw	0x30		;70
		retlw	0x04		;
		retlw	0x23		;
		retlw	0x08		;
		retlw	0x13		;74
		retlw	0x10		;
		retlw	0x02		;
		retlw	0x3C		;
		retlw	0x0C		;78
		retlw	0x1C		;
		retlw	0x33		;
		retlw	0x2C		;
		retlw	0x34		;7C
		retlw	0x12		;
		retlw	0x0E		;
		retlw	0x01		;
		retlw	0x2A		;80
		retlw	0x36		;
		retlw	0x20		;
		retlw	0x14		;
		retlw	0x24		;84
		retlw	0x37		;
		retlw	0x21		;
		retlw	0x17		;
		retlw	0x0C		;88
		retlw	0x34		;
		retlw	0x07		;
		retlw	0x02		;
		retlw	0x39		;8C
		retlw	0x0E		;
		retlw	0x35		;
		retlw	0x08		;
		retlw	0x13		;90
		retlw	0x11		;
		retlw	0x1A		;
		retlw	0x30		;
		retlw	0x3F		;94
		retlw	0x04		;
		retlw	0x30		;
		retlw	0x32		;
		retlw	0x09		;98
		retlw	0x27		;
		retlw	0x2D		;
		retlw	0x01		;
		retlw	0x2C		;9C
		retlw	0x19		;
		retlw	0x0C		;
		retlw	0x24		;
		retlw	0x3D		;A0
		retlw	0x04		;
		retlw	0x2B		;
		retlw	0x3E		;
		retlw	0x1D		;A4
		retlw	0x20		;
		retlw	0x10		;
		retlw	0x08		;
		retlw	0x38		;A8
		retlw	0x14		;
		retlw	0x3C		;
		retlw	0x30		;
		retlw	0x21		;AC
		retlw	0x10		;
		retlw	0x28		;
		retlw	0x0D		;
		retlw	0x03		;B0
		retlw	0x33		;
		retlw	0x18		;
		retlw	0x0A		;
		retlw	0x02		;B4
		retlw	0x20		;
		retlw	0x2E		;
		retlw	0x18		;
		retlw	0x40		;B8
		retlw	0x0B		;
		retlw	0x11		;
		retlw	0x32		;
		retlw	0x08		;BC
		retlw	0x3A		;
		retlw	0x0D		;
		retlw	0x29		;
		retlw	0x07		;C0
		retlw	0x31		;
		retlw	0x1C		;
		retlw	0x23		;
		retlw	0x1E		;C4
		retlw	0x03		;
		retlw	0x28		;
		retlw	0x0F		;
		retlw	0x1B		;C8
		retlw	0x23		;
		retlw	0x20		;
		retlw	0x0A		;
		retlw	0x06		;CC
		retlw	0x22		;
		retlw	0x3B		;
		retlw	0x0F		;
		retlw	0x38		;D0
		retlw	0x10		;
		retlw	0x12		;
		retlw	0x03		;
		retlw	0x13		;D4
		retlw	0x09		;
		retlw	0x31		;
		retlw	0x2F		;
		retlw	0x04		;D8
		retlw	0x06		;
		retlw	0x15		;
		retlw	0x10		;
		retlw	0x16		;DC
		retlw	0x08		;
		retlw	0x22		;
		retlw	0x10		;
		retlw	0x1F		;E0
		retlw	0x01		;
		retlw	0x0B		;
		retlw	0x03		;
		retlw	0x25		;E4
		retlw	0x0C		;
		retlw	0x26		;
		retlw	0x05		;
		retlw	0x30		;E8
		retlw	0x02		;
		retlw	0x05		;
		retlw	0xFF		;


 END












;     call   _flipLED   ;<--------------test code only-----------test code only------------------<<<<<<<
_flipLED    ;=====test program only-not part of project=========================
;flipLEd is used as a diagnostic tool, test output on selected pin to verify
;if the program actually reached the point where flipLED was inserted. This
;tool uses a spare port which must be set to output in the setup code.
 BTFSC	PORTA,3  ;PIC16F62X PIN 2
 GOTO	_flipOFF
 BSF	PORTA,3
 RETURN
_flipOFF
 BCF	PORTA,3
 RETURN

